"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const classes = require("../classes/index");
const helpers = require("../helpers/index");
const captureLogStream_1 = require("../helpers/captureLogStream");
const Node_1 = require("../../lib/classes/Node");
const util = require("../../lib/util");
const gracl_1 = require("../../lib/gracl");
const ava_1 = require("ava");
const permissionKey = 'graclPermissions';
const graph = new gracl_1.Graph({
    resources: [
        {
            permissionProperty: permissionKey,
            name: 'Post',
            parent: 'Blog',
            parentId: 'blogId',
            repository: classes.postModel
        },
        {
            permissionProperty: permissionKey,
            name: 'Blog',
            parent: 'Organization',
            parentId: 'organizationId',
            repository: classes.blogModel
        },
        {
            permissionProperty: permissionKey,
            name: 'Organization',
            repository: classes.orgModel
        }
    ],
    subjects: [
        {
            permissionProperty: permissionKey,
            name: 'User',
            parent: 'Team',
            parentId: 'teamIds',
            repository: classes.userModel
        },
        {
            permissionProperty: permissionKey,
            name: 'Team',
            parent: 'Organization',
            parentId: 'organizationId',
            repository: classes.teamModel
        },
        {
            permissionProperty: permissionKey,
            name: 'Organization',
            repository: classes.orgModel
        }
    ]
});
const graphClasses = {
    PostResource: graph.getResource('Post'),
    BlogResource: graph.getResource('Blog'),
    UserSubject: graph.getSubject('User'),
    TeamSubject: graph.getSubject('Team'),
    OrganizationResource: graph.getResource('Organization'),
    OrganizationSubject: graph.getSubject('Organization')
};
let orgA, orgB, teamA1, teamA2, teamA3, teamB1, userA1, userA2, userB1, blogA1, blogB1, postB1a, postB1b, postA1a;
ava_1.default.beforeEach(() => __awaiter(this, void 0, void 0, function* () {
    helpers.uidReset();
    // setup, add documents to models
    orgA = helpers.org();
    orgB = helpers.org();
    teamA1 = helpers.team(orgA);
    teamA2 = helpers.team(orgA);
    teamA3 = helpers.team(orgA);
    teamB1 = helpers.team(orgB);
    userA1 = helpers.user([teamA1, teamA2]);
    userA2 = helpers.user([teamA1, teamA3]);
    userB1 = helpers.user([teamB1]);
    blogA1 = helpers.blog(orgA);
    blogB1 = helpers.blog(orgB);
    postB1a = helpers.post(blogB1);
    postB1b = helpers.post(blogB1);
    postA1a = helpers.post(blogA1);
    yield Promise.all([
        classes.orgModel.saveEntity(orgA.id, orgA),
        classes.orgModel.saveEntity(orgB.id, orgB),
        classes.teamModel.saveEntity(teamA1.id, teamA1),
        classes.teamModel.saveEntity(teamA2.id, teamA2),
        classes.teamModel.saveEntity(teamA3.id, teamA3),
        classes.teamModel.saveEntity(teamB1.id, teamB1),
        classes.userModel.saveEntity(userA1.id, userA1),
        classes.userModel.saveEntity(userA2.id, userA2),
        classes.userModel.saveEntity(userB1.id, userB1),
        classes.blogModel.saveEntity(blogA1.id, blogA1),
        classes.blogModel.saveEntity(blogB1.id, blogB1),
        classes.postModel.saveEntity(postA1a.id, postA1a),
        classes.postModel.saveEntity(postB1a.id, postB1a),
        classes.postModel.saveEntity(postB1b.id, postB1b)
    ]);
}));
ava_1.default('yes() should return true', t => {
    t.true(util.yes());
});
ava_1.default('noop() should return void', t => {
    t.is(util.noop(), void 0);
});
ava_1.default('baseCompare() should correctly order items', t => {
    t.is(util.baseCompare('x', 'y'), -1, 'a < b  -> -1');
    t.is(util.baseCompare('y', 'x'), 1, 'a > b  -> 1');
    t.is(util.baseCompare('x', 'x'), 0, 'a == b -> 0');
});
ava_1.default('binaryIndexOf() should correctly find indexes on sorted arrays', t => {
    const A = ['a', 'b', 'c', 'd'], B = ['t', 'u', 'v', 'w', 'x', 'y', 'z'];
    t.is(util.binaryIndexOf(A, 'c'), 2, 'Correctly finds c in the array');
    t.is(util.binaryIndexOf(B, 'y'), 5, 'Correctly finds y in the array');
    t.is(util.binaryIndexOf(B, 'q'), -1, 'Returns -1 on not finding value');
});
ava_1.default('topologicalSort should correctly sort nodes', t => {
    const nodes = [
        { name: 'B', parent: 'A' },
        { name: 'A', parent: 'C' },
        { name: 'C' }
    ];
    t.deepEqual(util.topologicalSort(nodes), [
        { name: 'C' },
        { name: 'A', parent: 'C' },
        { name: 'B', parent: 'A' }
    ]);
});
ava_1.default('topologicalSort should throw when given a bad node', t => {
    const nodes = [
        { name: 'B', parent: 'A' },
        { name: 'A', parent: 'C' },
        { BAD_NODE: 'C' }
    ];
    t.throws(() => util.topologicalSort(nodes), /No name field on node/);
});
ava_1.default('topologicalSort should detect circular dependencies', t => {
    const nodes = [
        { name: 'A', parent: 'B' },
        { name: 'B', parent: 'A' }
    ];
    t.throws(() => util.topologicalSort(nodes), /Schema has a circular dependency or a missing parent/);
});
/**
 *
 * TODO: this is an odd es2015 classes bug...
 *
 */
// test.serial('Creating a node subclass without a repository should throw on instantiation', (t) => {
//   t.throws(() => {
//     class TestResource extends Resource {}
//     const result = new TestResource({});
//   }, 'Instantiate Resource');
//   t.throws(() => {
//     class TestSubject extends Subject {}
//     const result = new TestSubject({});
//   }, 'Instantiate Subject');
// });
ava_1.default.serial('Retrieving document from repository should work', (t) => __awaiter(this, void 0, void 0, function* () {
    t.is(yield classes.orgModel.getEntity(orgA.id), orgA, 'Memory Repository should sucessfully set items');
}));
ava_1.default.serial('Graph classes should have proper inheritance chain', t => {
    const { PostResource, BlogResource, UserSubject, TeamSubject, OrganizationResource, OrganizationSubject } = graphClasses;
    const PostResourceInstance = new PostResource({ id: 1 });
    const BlogResourceInstance = new BlogResource({ id: 1 });
    const UserSubjectInstance = new UserSubject({ id: 1 });
    const TeamSubjectInstance = new TeamSubject({ id: 1 });
    const OrganizationResourceInstance = new OrganizationResource({ id: 1 });
    const OrganizationSubjectInstance = new OrganizationSubject({ id: 1 });
    t.true(PostResourceInstance instanceof BlogResource, 'Post -> Blog');
    t.true(PostResourceInstance instanceof OrganizationResource, 'Post -> Org');
    t.true(UserSubjectInstance instanceof TeamSubject, 'User -> Team');
    t.true(UserSubjectInstance instanceof OrganizationSubject, 'User -> Org');
});
ava_1.default.serial('Graph classes return correct node depth', t => {
    const { BlogResource, UserSubject } = graphClasses;
    t.is(BlogResource.getNodeDepth(), 2);
    t.is(UserSubject.getNodeDepth(), 3);
});
ava_1.default.serial('Graph should throw if there is an undefined parent', t => {
    const createGraph = () => new gracl_1.Graph({
        resources: [
            {
                name: 'Post',
                parent: 'Blog',
                parentId: 'blogId',
                repository: classes.postModel
            },
            // no blog defined...
            // { name: 'Blog', parent: 'Organization', parentId: 'organizationId', repository: classes.blogModel },
            { name: 'Organization', repository: classes.orgModel }
        ],
        subjects: [
            {
                name: 'User',
                parent: 'Team',
                parentId: 'teamIds',
                repository: classes.userModel
            },
            {
                name: 'Team',
                parent: 'Organization',
                parentId: 'organizationId',
                repository: classes.teamModel
            },
            { name: 'Organization', repository: classes.orgModel }
        ]
    });
    t.throws(createGraph);
});
ava_1.default.serial('Graph should throw if there is a circular dependency', t => {
    const createGraph = () => new gracl_1.Graph({
        resources: [
            {
                name: 'Post',
                parent: 'Blog',
                parentId: 'blogId',
                repository: classes.postModel
            },
            {
                name: 'Blog',
                parent: 'Organization',
                parentId: 'organizationId',
                repository: classes.blogModel
            },
            { name: 'Organization', parent: 'Post', repository: classes.orgModel }
        ],
        subjects: [
            {
                name: 'User',
                parent: 'Team',
                parentId: 'teamIds',
                repository: classes.userModel
            },
            {
                name: 'Team',
                parent: 'Organization',
                parentId: 'organizationId',
                repository: classes.teamModel
            },
            { name: 'Organization', repository: classes.orgModel }
        ]
    });
    t.throws(createGraph);
});
ava_1.default.serial('Retrieving parent and child nodes from graph should succeed', t => {
    const UserSubject = graph.getSubject('User');
    const PostResource = graph.getResource('Post');
    const OrganizationResource = graph.getResource('Organization');
    const OrganizationSubject = graph.getResource('Organization');
    const resourceChildrenOfOrganization = graph.getChildResources(OrganizationResource);
    const subjectChildrenOfOrganization = graph.getChildSubjects(OrganizationSubject);
    const subjectParentsOfUser = graph.getParentSubjects(UserSubject);
    const resourceParentsOfPost = graph.getParentResources(PostResource);
    t.deepEqual(_.map(resourceChildrenOfOrganization, 'displayName'), [
        'Blog',
        'Post'
    ]);
    t.deepEqual(_.map(subjectChildrenOfOrganization, 'displayName'), [
        'Team',
        'User'
    ]);
    t.deepEqual(_.map(subjectParentsOfUser, 'displayName'), [
        'Team',
        'Organization'
    ]);
    t.deepEqual(_.map(resourceParentsOfPost, 'displayName'), [
        'Blog',
        'Organization'
    ]);
});
ava_1.default.serial(`Graph should throw when trying to retrieve classes that don't exist`, t => {
    t.throws(() => graph.getClass('DOESNT_EXIST', 'subject'), /No subject class found for DOESNT_EXIST/);
    t.throws(() => graph.getClass('User', 'INVALID'), /Invalid class type/);
});
ava_1.default.serial('Failed assertion that nodeClass is instanceof Node should throw', t => {
    t.throws(() => Node_1.Node.assertNodeClass({}), /is not an instance of Node/);
});
_.forEach([
    {
        description: 'user instantiated classes',
        classes: classes
    },
    {
        description: 'classes created by graph',
        classes: graphClasses
    }
], test => {
    runNodeTestsWithClasses(test.description, test.classes);
});
function runNodeTestsWithClasses(description, nodeClasses) {
    ava_1.default.serial(description + ' getRepository should return correct repository', t => {
        const resource = new nodeClasses.PostResource(postA1a);
        t.is(resource.getRepository(), nodeClasses.PostResource.repository);
    });
    ava_1.default.serial(description + ' isNodeType assertion should be correct', t => {
        const resource = new nodeClasses.PostResource(postA1a);
        t.is(resource.isNodeType(nodeClasses.PostResource), true);
        t.is(resource.isNodeType(nodeClasses.BlogResource), false);
    });
    ava_1.default.serial(description + ' Expect Resource.getNodeDepth to return correct depth', t => {
        const resource = new nodeClasses.PostResource(postA1a);
        t.is(resource.getNodeDepth(), 3);
    });
    ava_1.default.serial(description + ' Expect Resource.setDoc to populate permissions', (t) => __awaiter(this, void 0, void 0, function* () {
        const resource = new nodeClasses.PostResource(postA1a);
        delete postA1a['graclPermissions'];
        t.falsy(postA1a['graclPermissions']);
        resource.setDoc(postA1a);
        t.truthy(postA1a['graclPermissions']);
    }));
    ava_1.default.serial(description + ' getParentNode should return correct parent class', (t) => __awaiter(this, void 0, void 0, function* () {
        const resource = new nodeClasses.PostResource(postA1a);
        const parent = yield resource.getParentNode(blogA1);
        t.true(parent instanceof nodeClasses.BlogResource);
    }));
    ava_1.default.serial(description + ' getParents should warn if calling on root', (t) => __awaiter(this, void 0, void 0, function* () {
        const resource = new nodeClasses.OrganizationResource(orgA);
        const hook = captureLogStream_1.captureLogStream(process.stderr);
        const parents = yield resource.getParents();
        hook.unhook();
        t.regex(hook.captured(), /Calling Node.getParents()/);
    }));
    ava_1.default.serial(description + ' calling isAllowed without overriding should throw', (t) => __awaiter(this, void 0, void 0, function* () {
        const resource = new nodeClasses.OrganizationResource(orgA);
        const subject = new nodeClasses.OrganizationSubject(orgA);
        const isAllowed = nodeClasses.OrganizationResource.prototype.isAllowed;
        const resourceIsAllowed = gracl_1.Resource.prototype.isAllowed;
        delete gracl_1.Resource.prototype.isAllowed;
        delete nodeClasses.OrganizationResource.prototype.isAllowed;
        let threw = false;
        try {
            yield resource.isAllowed(subject, 'view');
        }
        catch (e) {
            t.regex(e.message, /Calling Node.isAllowed()/);
            threw = true;
        }
        t.true(threw);
        gracl_1.Resource.prototype.isAllowed = resourceIsAllowed;
        nodeClasses.OrganizationResource.prototype.isAllowed = isAllowed;
    }));
    ava_1.default.serial(description +
        ' getParentNode should throw if there is no available repository', (t) => __awaiter(this, void 0, void 0, function* () {
        const resource = new nodeClasses.PostResource(postA1a);
        const repository = nodeClasses.BlogResource.repository;
        nodeClasses.BlogResource.repository = false;
        let threw = false;
        try {
            yield resource.getParentNode('TEST');
        }
        catch (e) {
            t.regex(e.message, /No static repository property present on/);
            threw = true;
        }
        t.true(threw);
        nodeClasses.BlogResource.repository = repository;
    }));
    ava_1.default.serial(description +
        ' Resource.getParents() should return Resource instances of parent objects', (t) => __awaiter(this, void 0, void 0, function* () {
        const resource = new nodeClasses.PostResource(postA1a);
        const [parent] = yield resource.getParents();
        t.is(parent instanceof nodeClasses.BlogResource, true, 'Returned parent should be gracl node instance.');
        t.is(parent.getId(), blogA1.id, 'Correct parent should be returned.');
    }));
    ava_1.default.serial(description +
        ' Resource.allow(Subject, <perm>) -> subject can access resource.', (t) => __awaiter(this, void 0, void 0, function* () {
        const resource = new nodeClasses.PostResource(postA1a), subject = new nodeClasses.UserSubject(userA1);
        const initialAllowed = yield resource.isAllowed(subject, 'view');
        t.true((yield resource.allow(subject, 'view')) instanceof
            nodeClasses.PostResource, 'Setting permission should return same resource type.');
        const afterSetAllowed = yield resource.isAllowed(subject, 'view');
        t.false(initialAllowed, 'the subject should not yet be allowed to view the resource.');
        t.true(afterSetAllowed, 'After resource sets permission, they should have access');
    }));
    ava_1.default.serial(description + ' Resource and Subject methods should return same result', (t) => __awaiter(this, void 0, void 0, function* () {
        const resource = new nodeClasses.PostResource(postA1a), subject = new nodeClasses.UserSubject(userA1);
        yield resource.allow(subject, 'view');
        t.deepEqual(yield resource.isAllowed(subject, 'view'), yield subject.isAllowed(resource, 'view'), 'is allowed result should be the same');
        t.deepEqual(yield resource.determineAccess(subject, 'view'), yield subject.determineAccess(resource, 'view'), 'determine access result should be the same');
        t.deepEqual(yield resource.explainPermission(subject, 'view'), yield subject.explainPermission(resource, 'view'), 'explain permission result should be the same');
    }));
    ava_1.default.serial(description +
        ' Resource.allow(Subject, <perm>) -> resource should have permission with subjectType and resourceType set.', (t) => __awaiter(this, void 0, void 0, function* () {
        const resource = new nodeClasses.PostResource(postA1a), subject = new nodeClasses.UserSubject(userA1);
        yield resource.allow(subject, 'view');
        const [permission] = resource.doc[permissionKey];
        t.is(permission.resourceType, resource.getName());
        t.is(permission.subjectType, subject.getName());
    }));
    ava_1.default.serial(description +
        ' Resource.allow(Subject, <perm>) -> resource should not have permission with subjectType and resourceType set if failing assertionFn.', (t) => __awaiter(this, void 0, void 0, function* () {
        const resource = new nodeClasses.PostResource(postA1a), subject = new nodeClasses.UserSubject(userA1);
        yield resource.allow(subject, 'view');
        t.false(yield resource.isAllowed(subject, 'view', { assertionFn: () => false }));
    }));
    ava_1.default.serial(description +
        ' Resource.allow(parentSubject, <perm>) -> child subject can access resource.', (t) => __awaiter(this, void 0, void 0, function* () {
        const parentResource = new nodeClasses.BlogResource(blogA1), childResource = new nodeClasses.PostResource(postA1a), subject = new nodeClasses.UserSubject(userA1);
        const initialAllowed = yield childResource.isAllowed(subject, 'view');
        t.true((yield parentResource.allow(subject, 'view')) instanceof
            nodeClasses.BlogResource, 'Setting permission for parentSubject should return same resource type.');
        const afterSetAllowed = yield childResource.isAllowed(subject, 'view');
        t.false(initialAllowed, 'the child subject should not yet be allowed to view the resource.');
        t.true(afterSetAllowed, 'After resource sets permission, they should have access');
    }));
    ava_1.default.serial(description +
        ' parentResource.allow(parentSubject, <perm>) -> child subject can access child resource.', (t) => __awaiter(this, void 0, void 0, function* () {
        const parentResource = new nodeClasses.BlogResource(blogA1), childResource = new nodeClasses.PostResource(postA1a), parentSubject = new nodeClasses.TeamSubject(teamA1), childSubject = new nodeClasses.UserSubject(userA1);
        const initialAllowed = yield childResource.isAllowed(childSubject, 'view');
        t.true((yield parentResource.allow(parentSubject, 'view')) instanceof
            nodeClasses.BlogResource, 'Setting permission for parentSubject should return same resource type.');
        t.true((yield parentResource.allow(parentSubject, 'view')) instanceof
            nodeClasses.BlogResource, 'Calling allow twice should succeed');
        const afterSetAllowed = yield childResource.isAllowed(childSubject, 'view');
        t.false(initialAllowed, 'the child subject should not yet be allowed to view the resource.');
        t.true(afterSetAllowed, 'After resource sets permission, they should have access');
    }));
    ava_1.default.serial(description +
        ' Permissions should be visible through resource.getPermissionsHierarchy()', (t) => __awaiter(this, void 0, void 0, function* () {
        const parentResource = new nodeClasses.BlogResource(blogA1), childResource = new nodeClasses.PostResource(postA1a), subject = new nodeClasses.UserSubject(userA1);
        yield parentResource.allow(subject, 'view');
        // get hierarchy with childResource as root.
        const hiearchy = yield childResource.getPermissionsHierarchy();
        t.is(hiearchy.node, childResource.toString(), 'Node should be string representation.');
        t.is(hiearchy.parents[0].permissions.length, 1, 'Parent resource should have one permission.');
        t.true(hiearchy.parents[0].permissions[0].access['view'], 'View access should be true');
    }));
    /**
     *
        Post(Blog):
          - deny team access
        Blog:
          - allow team acces
        results:
          - user and team can access whole blog, except post
     */
    ava_1.default.serial(description +
        ' Lowest node on hierarchy wins conflicts (deny post for team)', (t) => __awaiter(this, void 0, void 0, function* () {
        const parentResource = new nodeClasses.BlogResource(blogA1), childResource = new nodeClasses.PostResource(postA1a), parentSubject = new nodeClasses.TeamSubject(teamA1), childSubject = new nodeClasses.UserSubject(userA1);
        t.false(yield childResource.isAllowed(childSubject, 'view'), 'User should not have access to post before permission set.');
        // allow team -> blog access
        yield parentResource.allow(parentSubject, 'view');
        // deny team specific access to post
        yield childResource.deny(parentSubject, 'view');
        t.true(yield parentResource.isAllowed(parentSubject, 'view'), 'Team should have access to blog after permission set');
        t.false(yield childResource.isAllowed(parentSubject, 'view'), 'Team should not have access to post after permission set.');
        t.true(yield parentResource.isAllowed(childSubject, 'view'), 'User should have access to blog after permission set');
        t.false(yield childResource.isAllowed(childSubject, 'view'), 'User should not have access to post after permission set.');
    }));
    /**
     *
        Post(Blog):
          - deny team access
          - allow user access
        Blog:
          - deny user access
          - allow team acces
        results:
          - user can access specific post, but not blog itself
          - team can access blog wholistically, but not specific post
     */
    ava_1.default.serial(description +
        ' Lowest node on hierarchy wins conflicts (deny post for team, but allow for user)', (t) => __awaiter(this, void 0, void 0, function* () {
        const parentResource = new nodeClasses.BlogResource(blogA1), childResource = new nodeClasses.PostResource(postA1a), parentSubject = new nodeClasses.TeamSubject(teamA1), childSubject = new nodeClasses.UserSubject(userA1);
        t.false(yield childResource.isAllowed(childSubject, 'view'), 'User should not have access to post before permission set.');
        // allow team -> blog access
        yield parentResource.allow(parentSubject, 'view');
        // deny team specific access to post
        yield childResource.deny(parentSubject, 'view');
        // allow child specifically to access post
        yield childResource.allow(childSubject, 'view');
        // deny child specifically to access blog
        yield parentResource.deny(childSubject, 'view');
        t.true(yield parentResource.isAllowed(parentSubject, 'view'), 'Team should have access to blog after permission set');
        t.false(yield childResource.isAllowed(parentSubject, 'view'), 'Team should not have access to post after permission set.');
        t.false(yield parentResource.isAllowed(childSubject, 'view'), 'User should not have access to blog after permission set');
        t.true(yield childResource.isAllowed(childSubject, 'view'), 'User should have access to post after permission set.');
    }));
    ava_1.default.serial(description +
        ' childResource.deny(parentSubject) should win over parentResource.allow(childSubject)', (t) => __awaiter(this, void 0, void 0, function* () {
        const parentResource = new nodeClasses.BlogResource(blogA1), childResource = new nodeClasses.PostResource(postA1a), parentSubject = new nodeClasses.TeamSubject(teamA1), childSubject = new nodeClasses.UserSubject(userA1);
        yield parentResource.allow(childSubject, 'view');
        yield childResource.deny(parentSubject, 'view');
        const access = yield childResource.isAllowed(childSubject, 'view');
        t.false(access, 'should not have access');
    }));
    ava_1.default.serial(description + ' Permission explainations should be accurate', (t) => __awaiter(this, void 0, void 0, function* () {
        const parentResource = new nodeClasses.BlogResource(blogA1), childResource = new nodeClasses.PostResource(postA1a), parentSubject = new nodeClasses.TeamSubject(teamA1), childSubject = new nodeClasses.UserSubject(userA1);
        // allow team -> blog access
        yield parentResource.allow(parentSubject, 'view');
        // deny team specific access to post
        yield childResource.deny(parentSubject, 'view');
        const reason = `Permission set on <Resource:${childResource.getName()} id=p0014> for <Subject:${parentSubject.getName()} id=t003> = false`;
        t.is(yield childResource.explainPermission(childSubject, 'view'), reason, 'Explaining why child subject cannot access child resource');
    }));
    ava_1.default.serial(description +
        ' Subject method results should equal resource method results', (t) => __awaiter(this, void 0, void 0, function* () {
        const parentResource = new nodeClasses.BlogResource(blogA1), childResource = new nodeClasses.PostResource(postA1a), parentSubject = new nodeClasses.TeamSubject(teamA1), childSubject = new nodeClasses.UserSubject(userA1);
        // allow team -> blog access
        yield parentResource.allow(parentSubject, 'view');
        // deny team specific access to post
        yield childResource.deny(parentSubject, 'view');
        t.is(yield parentResource.isAllowed(parentSubject, 'view'), yield parentSubject.isAllowed(parentResource, 'view'), 'Team should have access to blog after permission set');
        t.is(yield childResource.isAllowed(parentSubject, 'view'), yield parentSubject.isAllowed(childResource, 'view'), 'Team should not have access to post after permission set.');
        t.is(yield parentResource.isAllowed(childSubject, 'view'), yield childSubject.isAllowed(parentResource, 'view'), 'User should have access to blog after permission set');
        t.is(yield childResource.isAllowed(childSubject, 'view'), yield childSubject.isAllowed(childResource, 'view'), 'User should have access to post after permission set.');
    }));
    ava_1.default.serial(description +
        ' Node.getHierarchyIds() should return flattened array of correct ids', (t) => __awaiter(this, void 0, void 0, function* () {
        const childResource = new nodeClasses.PostResource(postA1a);
        t.deepEqual(yield childResource.getHierarchyIds(), ['p0014', 'b0010', 'o001'], 'Post -> Blog -> Org');
    }));
    ava_1.default.serial(description +
        ' Node.getHierarchyClassNames() should return array of class names', t => {
        const childResource = new nodeClasses.PostResource(postA1a), childSubject = new nodeClasses.UserSubject(userA1);
        const name = (c) => (c.displayName || c.name);
        t.deepEqual(childResource.getHierarchyClassNames(), [
            name(nodeClasses.PostResource),
            name(nodeClasses.BlogResource),
            name(nodeClasses.OrganizationResource)
        ], 'resource class names');
        t.deepEqual(childSubject.getHierarchyClassNames(), [
            name(nodeClasses.UserSubject),
            name(nodeClasses.TeamSubject),
            name(nodeClasses.OrganizationSubject)
        ], 'subject class names');
    });
    ava_1.default.serial(description + ' Should get access through parent having access to itself', (t) => __awaiter(this, void 0, void 0, function* () {
        const parentResource = new nodeClasses.OrganizationResource(orgA), parentSubject = new nodeClasses.OrganizationSubject(orgA), childSubject = new nodeClasses.UserSubject(userA1);
        // allow team -> blog access
        yield parentResource.allow(parentSubject, 'view');
        const perm = yield parentResource.determineAccess(childSubject, 'view');
        t.true(perm['view'].access, 'child should have access through parent -> parent');
    }));
    /**
     * say a user has two teams, and one of those teams is denied access
       to a given component, while the other team is allowed access to the same
       component -- gracl should return deny. Additionally, the same
       logic should apply if there are multiple resources with conflicting access
     */
    ava_1.default.serial(description +
        ' multiple resogetParentResources with conflicting access at the same hierarchy depth should default to deny', (t) => __awaiter(this, void 0, void 0, function* () {
        const subject = new nodeClasses.UserSubject(userA1);
        const resource = new nodeClasses.PostResource(postA1a);
        const [parentSubject1, parentSubject2] = yield subject.getParents();
        yield resource.deny(parentSubject2, 'view');
        yield resource.allow(parentSubject1, 'view');
        t.false(yield resource.isAllowed(subject, 'view'));
    }));
    ava_1.default.serial(description + ' checking multiple perms should short circuit correctly', (t) => __awaiter(this, void 0, void 0, function* () {
        const subject = new nodeClasses.UserSubject(userA1);
        const resource = new nodeClasses.PostResource(postA1a);
        const [parentSubject1, parentSubject2] = yield subject.getParents();
        yield resource.deny(parentSubject2, 'deniedPerm');
        yield resource.allow(parentSubject1, 'deniedPerm');
        yield resource.allow(subject, 'allowedPerm');
        const accessResults = yield resource.determineAccess(subject, [
            'deniedPerm',
            'allowedPerm'
        ]);
        t.false(accessResults['deniedPerm'].access, 'conflicting parent perm should be false after short circuit');
        t.true(accessResults['allowedPerm'].access, 'perm with no conflict should be true');
    }));
}
